#!/usr/bin/env python
# coding=utf-8

import json
import time
import xml.etree.ElementTree as et
import os
from progress.bar import IncrementalBar
from progress.spinner import Spinner
import pymysql

import parapheur  # Configuration
from parapheur.parapheur import config
from parapheur.parapheur import pprint  # Colored printer

targetdir = config.get("Parapheur", "exportdir")

# Init REST API client
client = parapheur.getrestclient()

# Init database connexion
cnx = pymysql.connect(user=config.get("Database", "username"), password=config.get("Database", "password"),
                      host=config.get("Database", "server"), database=config.get("Database", "database"))

pprint.header("\nExport de la configuration du parapheur en cours ...")

if not os.path.exists(targetdir):
    os.makedirs(targetdir)
os.chdir(targetdir)


def handle_export(kind, func):
    # Handle main dir
    maindir = kind
    if not os.path.exists(maindir):
        os.makedirs(maindir)
    os.chdir(maindir)

    # Here, do stuff
    func()

    # Quit main dir
    os.chdir("..")


def user_handler():

    # Build directories structure
    os.makedirs("prefs")
    os.makedirs("sigs")
    os.makedirs("certs")
    os.makedirs("admins")

    # Get all users
    users = client.doget("/parapheur/utilisateurs")
    md4_passwords = {}

    bar = IncrementalBar('Export Utilisateurs', max=len(users), suffix='%(index)d/%(max)d - %(eta)ds')

    with open('list.json', 'w') as outfile:
        json.dump(users, outfile)

    # Get each user preferences, sig image and certificate
    for i, user in enumerate(users):
        encoded_username = user['username'].encode('utf-8')
        # Preferences
        userpref = client.doget("/api/people/%s/preferences" % encoded_username)
        with open('prefs/%s.json' % encoded_username, 'w') as outfile:
            json.dump(userpref, outfile)

        # Images
        singleuser = client.doget("/parapheur/utilisateurs/%s" % user['id'])
        if "signature" in singleuser:
            client.dodownload("/api/node/workspace/SpacesStore/%s/content" % singleuser['signature'],
                              "sigs/%s.png" % encoded_username)

        # Certificates
        if "certificat" in singleuser:
            # https://parapheur.lh.dev.libriciel.fr/iparapheur/proxy/alfresco/api/node/content%3bph%3acertificat/workspace/SpacesStore/d8a4e83b-889b-4e86-abe9-86fb5eca34f6/Delphine%20LEROUX.p12
            client.dodownload(
                "/api/node/content%%3bph%%3acertificat/workspace/SpacesStore/%s/%s" % (user['id'], encoded_username),
                "certs/%s.cer" % encoded_username)

        # Get admin fonctionnel informations
        if "admin" in singleuser and singleuser["admin"] == "adminFonctionnel":
            administres = client.doget("/parapheur/utilisateurs/%s/bureaux" % user['id'], {"administres": "true"})
            with open('admins/%s.json' % encoded_username, 'w') as outfile:
                json.dump(administres, outfile)

        cursor = cnx.cursor()
        query = "SELECT anp1.string_value " \
                "FROM alf_node_properties anp1 " \
                "INNER JOIN alf_qname aq1 " \
                "ON aq1.id = anp1.qname_id " \
                "INNER JOIN alf_node_properties anp2 " \
                "ON anp2.node_id = anp1.node_id " \
                "INNER JOIN alf_qname aq2 " \
                "ON aq2.id = anp2.qname_id " \
                "WHERE aq1.local_name    = 'password'  " \
                "AND aq2.local_name  = 'username' " \
                "AND anp2.string_value = '%s';" % (user['username'].encode('utf-8'))
        cursor.execute(query)
        for res in cursor:
            md4_passwords[user['username'].encode('utf-8')] = res[0]

        bar.next()

    with open('passwords.json', 'w') as outfile:
        json.dump(md4_passwords, outfile)

    bar.finish()
    time.sleep(0.1)


def groups_handler():

    # Get all groups
    groups = client.doget("/parapheur/groupes")

    bar = IncrementalBar('Export Groupes     ', max=len(groups), suffix='%(index)d/%(max)d - %(eta)ds')

    with open('list.json', 'w') as outfile:
        json.dump(groups, outfile)

    # Get detail for each group
    for i, group in enumerate(groups):
        detail_group = client.doget("/parapheur/groupes/%s" % group["id"])
        with open('%s.json' % group["id"], 'w') as outfile:
            json.dump(detail_group, outfile)

        bar.next()

    bar.finish()
    time.sleep(0.1)


def bureaux_handler():

    # Get all bureaux
    bureaux = client.doget("/parapheur/bureaux", {"asAdmin": "true"})

    bar = IncrementalBar('Export Bureaux     ', max=len(bureaux), suffix='%(index)d/%(max)d - %(eta)ds')

    for i, bureau in enumerate(bureaux):
        response = client.doget("/parapheur/bureaux/%s/associes" % bureau['id'], {"asAdmin": "true"})
        if 'delegations-possibles' in response:
            bureau['delegations-possibles'] = response['delegations-possibles']
            bureaux[i] = bureau

        bar.next()

    with open('list.json', 'w') as outfile:
        json.dump(bureaux, outfile)

    bar.finish()
    time.sleep(0.1)


def circuits_handler():

    spinner = Spinner('Export Circuits ...')

    spinner.next()
    # Get all circuits
    circuits = client.doget("/parapheur/circuits")

    with open('list.json', 'w') as outfile:
        json.dump(circuits, outfile)


def types_handler():

    # Get all types
    types = client.doget("/parapheur/types", {"asAdmin": "true"})

    with open('list.json', 'w') as outfile:
        json.dump(types, outfile)

    bar = IncrementalBar('Export Types       ', max=len(types), suffix='%(index)d/%(max)d - %(eta)ds')

    # For each type, get all subtypes
    for i, typo in enumerate(types):
        # There is PAdES configuration... We need to get it back !
        if 'PAdES' in typo['sigFormat']:
            pades_config = client.doget("/parapheur/types/%s/overridePades" % typo["id"].encode('utf-8'))
            with open('%s_pades.json' % typo["id"].encode('utf-8'), 'w') as outfile:
                json.dump(pades_config, outfile)
        if 'ACTES' in typo['tdtProtocole']:
            actes_config = client.doget("/parapheur/types/%s/overrideActes" % typo["id"].encode('utf-8'))
            with open('%s_actes.json' % typo["id"].encode('utf-8'), 'w') as outfile:
                json.dump(actes_config, outfile)
        if 'HELIOS' in typo['tdtProtocole']:
            helios_config = client.doget("/parapheur/types/%s/overrideHelios" % typo["id"].encode('utf-8'))
            with open('%s_helios.json' % typo["id"].encode('utf-8'), 'w') as outfile:
                json.dump(helios_config, outfile)
        # Create path for typo
        os.makedirs(typo["id"])
        os.chdir(typo["id"])
        for subtype in typo["sousTypes"]:
            subtype_info = client.doget(
                "/parapheur/types/%s/%s" % (subtype["parent"].encode('utf-8'), subtype["id"].encode('utf-8')))
            with open('%s.json' % subtype["id"].encode('utf-8'), 'w') as outfile:
                json.dump(subtype_info, outfile)
        os.chdir("..")

        bar.next()

    bar.finish()
    time.sleep(0.1)


def metadata_hanlder():

    spinner = Spinner('Export Metadonnees ...')

    spinner.next()

    metadatas = client.doget("/parapheur/metadonnees", {"asAdmin": "true"})

    with open('list.json', 'w') as outfile:
        json.dump(metadatas, outfile)


def calque_handler():

    calques = client.doget("/parapheur/calques")

    with open('list.json', 'w') as outfile:
        json.dump(calques, outfile)

    bar = IncrementalBar('Export Calques     ', max=len(calques), suffix='%(index)d/%(max)d - %(eta)ds')

    for i, calque in enumerate(calques):
        calque_encoded = calque['id']
        if not os.path.exists(calque_encoded):
            os.mkdir(calque_encoded)
            os.chdir(calque_encoded)

            # do stuffs like getting all infos
            signature = client.doget("/parapheur/calques/%s/signature" % calque['id'])
            with open('signature.json', 'w') as outfile:
                json.dump(signature, outfile)

            images = client.doget("/parapheur/calques/%s/image" % calque['id'])
            with open('image.json', 'w') as outfile:
                json.dump(images, outfile)
            # for each image get the actual file in base
            for image in images:
                imagename = image['nomImage'].encode('utf-8')
                client.dodownloadfromnode("/workspace/SpacesStore/%s/file.bin" % image['id'],
                                          "{http://www.starxpert.fr/alfresco/model/calque/1.0}contenuImage",
                                          "%s" % imagename)

            commentaire = client.doget("/parapheur/calques/%s/commentaire" % calque['id'])
            with open('commentaire.json', 'w') as outfile:
                json.dump(commentaire, outfile)
            metadata = client.doget("/parapheur/calques/%s/metadata" % calque['id'])
            with open('metadata.json', 'w') as outfile:
                json.dump(metadata, outfile)

            os.chdir('..')

        bar.next()

    bar.finish()
    time.sleep(0.1)


def advanced_handler():

    spinner = Spinner('Export Conf. avancee ...')

    os.mkdir("certs")

    # https://parapheur.test.adullact.org/iparapheur/proxy/alfresco/slingshot/doclib/treenode/node/alfresco/company/home/Dictionnaire%20de%20donn%C3%A9es/Certificats
    # Get certificates noderef
    response = client.doget(
        '/slingshot/doclib/treenode/node/alfresco/company/home/Dictionnaire%20de%20données/Certificats')

    spinner.next()

    certs_id = response["parent"]["nodeRef"].split('/')[-1]
    response = client.doget("/cmis/s/workspace:SpacesStore/i/%s/children" % certs_id)

    spinner.next()

    # Parse the XML result
    root_tag = et.ElementTree(et.fromstring(response)).getroot()
    for child in root_tag.iter("{http://www.w3.org/2005/Atom}entry"):
        content_url = child.find("{http://www.w3.org/2005/Atom}content").get('src').split('alfresco/wcs')[1]
        content_title = child.find("{http://www.w3.org/2005/Atom}title").text

        if content_url.endswith('.p12'):
            client.dodownload(content_url, "certs/%s" % content_title.encode('utf-8'))
            spinner.next()

    actes_infos = client.doget("/parapheur/connecteurs/s2low/actes")
    with open('actes.json', 'w') as outfile:
        json.dump(actes_infos, outfile)

    spinner.next()

    helios_infos = client.doget("/parapheur/connecteurs/s2low/helios")
    with open('helios.json', 'w') as outfile:
        json.dump(helios_infos, outfile)

    spinner.next()

    mailsec_infos = client.doget("/parapheur/connecteurs/s2low/mailsec")
    with open('mailsec.json', 'w') as outfile:
        json.dump(mailsec_infos, outfile)

    spinner.next()


handle_export("users", user_handler)
handle_export("groups", groups_handler)
handle_export("bureaux", bureaux_handler)
handle_export("circuits", circuits_handler)
handle_export("types", types_handler)
handle_export("metadatas", metadata_hanlder)
handle_export("calques", calque_handler)
handle_export("advanced", advanced_handler)



#!/usr/bin/env python
# coding=utf-8

import copy
import json
import time

import os
import pymysql
from parapheur.parapheur import config
from progress.bar import IncrementalBar

import parapheur  # Configuration

importdir = config.get("Parapheur", "importdir")
os.chdir(importdir)

# Init REST API client
client = parapheur.getrestclient()

# Init database connexion
cnx = pymysql.connect(user=config.get("Database", "username"), password=config.get("Database", "password"),
                      host=config.get("Database", "server"), database=config.get("Database", "database"))


class ParapheurImportTemplate(object):
    """
    This is the template for importing data in parapheur
    """

    def __init__(self, _importdir, _name, _client):
        self.importdir = _importdir
        self.client = _client
        self.name = _name
        self.launch()

    def __before_func__(self):
        print("This have to be implemented - Return list of elements")
        return []

    def __each_func__(self, element):
        print("This have to be implemented - Return updated element")
        return element

    def __after_func__(self):
        print("This have to be implemented")

    def launch(self):
        """
        This method is just a template for importing things properly ...
        :return: nothing, it just logs a progressbar
        """

        list_to_handle = self.__before_func__()

        bar = IncrementalBar('Importing %s ...' % self.name.encode('utf-8'), max=len(list_to_handle),
                             suffix='%(index)d/%(max)d - %(eta)ds')

        for i, element in enumerate(list_to_handle):
            list_to_handle[i] = self.__each_func__(element)
            bar.next()

        self.__after_func__()
        bar.finish()
        time.sleep(0.1)


class UsersImport(ParapheurImportTemplate):
    def __before_func__(self):
        self.users = None
        with open("users/list.json") as data_file:
            self.users = json.load(data_file)
        # Load passwords
        self.passwords = {}
        with open("users/passwords.json") as data_file:
            self.passwords = json.load(data_file)
        # Get all users
        self.users_exists = client.doget("/parapheur/utilisateurs")
        return self.users

    def __each_func__(self, element):
        encoded_username = element['username'].encode('utf-8')
        # 1 - Create users
        element['password'] = "secret1"
        if "firstName" not in element:
            element["firstName"] = ""
        if "lastName" not in element:
            element["lastName"] = ""
        if "email" not in element:
            element["email"] = ""
        response = client.dopost("/parapheur/utilisateurs", element)
        # Si l'utilisateur n'existe pas, on le crée
        if response['id'] != "already exists":
            element['id'] = response['id']
        # Sinon, on récupère l'id existant
        else:
            user_exists = (x for x in self.users_exists if x['username'] == element['username']).next()
            element['id'] = user_exists['id']

        # 2 - Update all properties + sigs + cert
        if os.path.isfile("users/sigs/%s.png" % encoded_username):
            # We have a sig image !
            element['signatureData'] = client.load_binary_file("users/sigs/%s.png" % encoded_username)
        if os.path.isfile("users/certs/%s.cer" % encoded_username):
            # We have a certificate !
            element['certificat'] = {
                "content": client.load_binary_file("users/certs/%s.cer" % encoded_username)
            }

        client.doput("/parapheur/utilisateurs/%s" % element['id'], element)

        # Import preferences
        with open("users/prefs/%s.json" % encoded_username) as data_file:
            prefs = json.load(data_file)
            client.dodelete("/api/people/%s/preferences" % encoded_username)
            client.dopost("/api/people/%s/preferences" % encoded_username, prefs)

        # In some cases, user does not have a password
        if encoded_username in self.passwords:
            # We have to get node_id and qname_id of user before updating password...
            cursor = cnx.cursor()
            query = "SELECT anp1.node_id, anp1.qname_id " \
                    "FROM alf_node_properties anp1 " \
                    "INNER JOIN alf_qname aq1 " \
                    "ON aq1.id = anp1.qname_id " \
                    "INNER JOIN alf_node_properties anp2 " \
                    "ON anp2.node_id = anp1.node_id " \
                    "INNER JOIN alf_qname aq2 " \
                    "ON aq2.id = anp2.qname_id " \
                    "WHERE aq1.local_name    = 'password'  " \
                    "AND aq2.local_name  = 'username' " \
                    "AND anp2.string_value = '%s';" % encoded_username
            cursor.execute(query)
            for res in cursor:
                cursor = cnx.cursor()
                # We only have one result !
                updating_passord_query = "UPDATE alf_node_properties " \
                                         "SET string_value='%s' " \
                                         "WHERE node_id=%s " \
                                         "AND qname_id=%s;" % (self.passwords[encoded_username], res[0], res[1])
                cursor.execute(updating_passord_query)
                # Actually commit the transaction
                cnx.commit()
        return element

    def __after_func__(self):
        pass


class GroupsImport(ParapheurImportTemplate):
    def __before_func__(self):
        self.groups = None

        with open("groups/list.json") as data_file:
            self.groups = json.load(data_file)

        # Get all groups
        self.groups_exists = client.doget("/parapheur/groupes")
        # Keep the old ones for id references
        self.old_groups = copy.deepcopy(self.groups)

        return self.groups

    def __each_func__(self, element):
        # complete group
        with open("groups/%s.json" % element['id']) as data_file:
            groups_detail = json.load(data_file)
            element.update(groups_detail)
        response = client.dopost("/parapheur/groupes", element)
        if "status" not in response:
            # group created
            element['id'] = response['id']
        else:
            # group exist
            group_exists = (x for x in self.groups_exists if x['shortName'] == element['shortName']).next()
            element['id'] = group_exists['id']

        for member in element['users']:
            # Add each member to group
            client.dopost("/parapheur/groupes/%s/%s" % (element['id'], member['shortName'].encode('utf-8')))
        return element

    def __after_func__(self):
        # We have to build dict with old and new groups ids
        self.group_ids_relation = {}
        for group in self.groups:
            old_b = (b for b in self.old_groups if b['shortName'] == group['shortName']).next()
            self.group_ids_relation[old_b['id']] = group['id']


class MetasImport(ParapheurImportTemplate):
    def __before_func__(self):
        with open("metadatas/list.json") as data_file:
            metadatas = json.load(data_file)
        return metadatas

    def __each_func__(self, element):
        client.dopost("/parapheur/metadonnees/%s" % element['id'], element)
        return element

    def __after_func__(self):
        pass


class BureauxImport(ParapheurImportTemplate):
    def __before_func__(self):
        self.bureaux = None
        with open("bureaux/list.json") as data_file:
            self.bureaux = json.load(data_file)

        # Get all bureaux
        self.bureaux_exists = client.doget("/parapheur/bureaux", {"asAdmin": "true"})
        # Keep the old ones for id references
        self.old_bureaux = copy.deepcopy(self.bureaux)
        return self.bureaux

    def __each_func__(self, element):
        # Ugly hack for bureaux random errors
        while True:
            if not any(x['name'] == element['name'] for x in self.bureaux_exists):
                # We have to remove "null" habilitations
                for k, v in element.items():
                    if v is None:
                        del element[k]
                try:
                    response = client.dopost("/parapheur/bureaux", element)
                    element['id'] = response['id']
                    break
                except:
                    # Refresh local bureaux
                    self.bureaux_exists = client.doget("/parapheur/bureaux", {"asAdmin": "true"})
            else:
                bureau_exists = (x for x in self.bureaux_exists if x['name'] == element['name']).next()
                element['id'] = bureau_exists['id']
                break
        return element

    def __after_func__(self):
        # We have to build dict with old and new bureaux ids
        self.bureau_ids_relation = {}
        for bureau in self.bureaux:
            old_b = (b for b in self.old_bureaux if b['name'] == bureau['name']).next()
            self.bureau_ids_relation[old_b['id']] = bureau['id']


class BureauxAdvancedImport(ParapheurImportTemplate):
    def __init__(self, _importdir, _client, _users, _bureaux, _bureau_ids_relation):
        self.bureaux = _bureaux
        self.users = _users
        self.bureau_ids_relation = _bureau_ids_relation
        super(BureauxAdvancedImport, self).__init__(_importdir, "advanced bureaux configuration", _client)

    def __before_func__(self):
        return self.bureaux

    def __each_func__(self, element):
        element['delegations-possibles'] = [self.bureau_ids_relation[x] for x in element['delegations-possibles']]
        to_update = {'delegations-possibles': element['delegations-possibles']}

        if element["hierarchie"]:
            element['hierarchie'] = self.bureau_ids_relation[element['hierarchie']]
            to_update['hierarchie'] = element['hierarchie']

        client.doput("/parapheur/bureaux/%s" % element['id'], to_update)

        if "idCible" in element["delegation"]:
            element['delegation']["idCible"] = self.bureau_ids_relation[element['delegation']["idCible"]]
            client.doput("/parapheur/delegations/%s" % element['id'], element['delegation'])

        return element

    def __after_func__(self):
        for i, user in enumerate(self.users):
            if os.path.isfile("users/admins/%s.json" % user['username'].encode('utf-8')):
                with open("users/admins/%s.json" % user['username'].encode('utf-8'), "r") as json_file:
                    # Remove duplicates
                    administres = list(set(json.load(json_file)))
                    # Correct ids of bureaux
                    administres = [self.bureau_ids_relation[x] for x in administres]
                    client.doput("/parapheur/utilisateurs/%s" % user['id'], {
                        "admin": "adminFonctionnel",
                        "bureauxAdministres": administres,
                        "isAdmin": False,
                        "isAdminFonctionnel": True
                    })


class CircuitsImport(ParapheurImportTemplate):
    def __init__(self, _importdir, _client, _bureau_ids_relation):
        self.bureau_ids_relation = _bureau_ids_relation
        super(CircuitsImport, self).__init__(_importdir, "circuits", _client)

    def __before_func__(self):
        with open("circuits/list.json") as data_file:
            circuits = json.load(data_file)

        # Get all users
        self.circuits_exists = client.doget("/parapheur/circuits")

        return circuits

    def __each_func__(self, element):
        # Replace all occurence of bureau id in each step
        for j, step in enumerate(element['etapes']):
            if "parapheur" in step:
                step['parapheur'] = self.bureau_ids_relation[step['parapheur']]
                step['listeNotification'] = [self.bureau_ids_relation[x] if x != "_emetteur_" else "_emetteur_" for x in
                                             step['listeNotification']]
            element['etapes'][j] = step

        client.dopost("/parapheur/circuits", element)

        return element

    def __after_func__(self):
        pass


class TyposImports(ParapheurImportTemplate):
    def __init__(self, _importdir, _client, _bureau_ids_relation, _group_ids_relation, _calque_ids_relation):
        self.bureau_ids_relation = _bureau_ids_relation
        self.group_ids_relation = _group_ids_relation
        self.calque_ids_relation = _calque_ids_relation

        super(TyposImports, self).__init__(_importdir, "typologies", _client)

    def __before_func__(self):
        with open("types/list.json") as data_file:
            types = json.load(data_file)
        return types

    def __each_func__(self, element):
        # Create type
        client.dopost("/parapheur/types/%s" % element['id'].encode('utf-8'), element)

        # Eventually do pades thing
        if os.path.isfile("types/%s_pades.json" % element['id'].encode('utf-8')):
            with open("types/%s_pades.json" % element['id'].encode("utf-8")) as data_file:
                pades_info = json.load(data_file)
                client.doput("/parapheur/types/%s/overridePades" % element['id'].encode("utf-8"),
                             pades_info['overridedTdt'])
        # Eventually do actes thing
        if os.path.isfile("types/%s_actes.json" % element['id'].encode('utf-8')):
            with open("types/%s_actes.json" % element['id'].encode("utf-8")) as data_file:
                actes_info = json.load(data_file)['overridedTdt']
                if os.path.isfile("advanced/certs/%s" % actes_info['name'].encode('utf-8')):
                    base64cert = client.load_binary_file("advanced/certs/%s" % actes_info['name'].encode('utf-8'))
                    actes_info['cert'] = base64cert
                    client.doput("/parapheur/types/%s/overrideActes" % element['id'].encode("utf-8"),
                                 actes_info)
        # Eventually do helios thing
        if os.path.isfile("types/%s_helios.json" % element['id'].encode('utf-8')):
            with open("types/%s_helios.json" % element['id'].encode("utf-8")) as data_file:
                helios_info = json.load(data_file)['overridedTdt']
                if os.path.isfile("advanced/certs/%s" % helios_info['name'].encode('utf-8')):
                    base64cert = client.load_binary_file("advanced/certs/%s" % helios_info['name'].encode('utf-8'))
                    helios_info['cert'] = base64cert
                    client.doput("/parapheur/types/%s/overrideHelios" % element['id'].encode("utf-8"),
                                 helios_info)

        # Create all subtypes
        for subtype in element['sousTypes']:
            with open("types/%s/%s.json" % (element['id'].encode("utf-8"), subtype['id'].encode("utf-8"))) as data_file:
                subtype_info = json.load(data_file)
            # Correct ids for bureaux
            subtype_info['parapheurs'] = [self.bureau_ids_relation[x] for x in subtype_info['parapheurs']]
            subtype_info['parapheursFilters'] = [self.bureau_ids_relation[x] for x in subtype_info['parapheursFilters']]
            # Correct ids for groups
            try:
                subtype_info['groups'] = [self.group_ids_relation[x] if x in self.group_ids_relation else x for x in
                                          subtype_info['groups']]
            except:
                # ignore
                print("Error : cannot find group")
            try:
                subtype_info['groupsFilters'] = [self.group_ids_relation[x] if x in self.group_ids_relation else x for x
                                                 in subtype_info['groupsFilters']]
            except:
                # ignore
                print("Error : cannot find group")
            # Correct ids for calques
            for j, calque in enumerate(subtype_info['calques']):
                calque['id'] = self.calque_ids_relation[calque['id']]
                subtype_info['calques'][j] = calque

            # Create subtype
            client.dopost(
                "/parapheur/types/%s/%s" % (element['id'].encode("utf-8"), subtype['id'].encode("utf-8")), subtype_info)
        return element

    def __after_func__(self):
        pass


class CalquesImport(ParapheurImportTemplate):
    def __before_func__(self):
        with open("calques/list.json") as data_file:
            self.calques = json.load(data_file)

        # Get all users
        self.circuits_exists = client.doget("/parapheur/circuits")
        self.calque_ids_relation = {}

        return self.calques

    def __each_func__(self, element):
        # Create the calque
        response = client.dopost("/parapheur/calques", element)

        old_id = element['id']
        self.calque_ids_relation[element['id']] = response['id']
        element['id'] = response['id']

        def loadmetaobj(name, police_number=False):
            # Then create all subelements
            with open("calques/%s/%s.json" % (old_id, name)) as data_file:
                elements = json.load(data_file)
                for com in elements:
                    if com['postSignature'] is None:
                        com['postSignature'] = False
                    for k, v in com.items():
                        if isinstance(v, int) and not isinstance(v, bool):
                            com[k] = str(v)
                    if police_number:
                        com['taillePolice'] = int(com['taillePolice'])
                    if "nomImage" in com:
                        binary_img = client.load_binary_file(
                            "calques/%s/%s" % (old_id, com['nomImage'].encode('utf-8')))
                        com['fichierImage'] = binary_img
                    client.dopost("/parapheur/calques/%s/%s" % (element['id'], name), com)

        loadmetaobj("commentaire")
        loadmetaobj("metadata", True)
        loadmetaobj("signature")
        loadmetaobj("image")

        return element

    def __after_func__(self):
        pass


class AdvancedImport(ParapheurImportTemplate):
    def __before_func__(self):
        return ['actes', 'helios', 'mailsec']

    def __each_func__(self, element):
        with open("advanced/%s.json" % element) as data_file:
            data = json.load(data_file)
            # Load cert file if exist
            if os.path.isfile("advanced/certs/%s" % data['name'].encode('utf-8')):
                base64cert = client.load_binary_file("advanced/certs/%s" % data['name'].encode('utf-8'))
                data['cert'] = base64cert
                client.doput("/parapheur/connecteurs/s2low/%s" % element, data)

        return element

    def __after_func__(self):
        pass


# Classic user import
importedUsers = UsersImport(importdir, "users", client)
# Groups import, keep saved groups for ids relationship
importedGroups = GroupsImport(importdir, "groups", client)
# Classic metas import
MetasImport(importdir, "metadatas", client)
# Classic calques import
importedCalques = CalquesImport(importdir, "calques", client)
# Bureaux import, keep saved ones for ids relationship
importedBureaux = BureauxImport(importdir, "bureaux", client)
# Advanced configuration import with previously saved infos
BureauxAdvancedImport(importdir, client, importedUsers.users, importedBureaux.bureaux,
                      importedBureaux.bureau_ids_relation)
# Circuits import with previously saved infos
CircuitsImport(importdir, client, importedBureaux.bureau_ids_relation)
# Typo import with bureaux and groups ids relationship
TyposImports(importdir, client, importedBureaux.bureau_ids_relation, importedGroups.group_ids_relation,
             importedCalques.calque_ids_relation)
# Import Actes, Helios and mailsec configuration
AdvancedImport(importdir, "advanced configuration", client)



# coding=utf-8

import sys
import io

from parapheur.parapheur import config  # Configuration
from parapheur.parapheur import pprint  # Colored printer

req_version = (3, 0)
cur_version = sys.version_info
isp3 = cur_version >= req_version
# pprint.log(cur_version)

if isp3:
    # noinspection PyCompatibility,PyUnresolvedReferences
    import configparser as ConfigParser
else:
    # noinspection PyCompatibility
    import ConfigParser

ALF_CONFIG_PATH = "/opt/iParapheur/tomcat/shared/classes/alfresco-global.properties"

with open(ALF_CONFIG_PATH, 'r') as f:
    config_string = '[Parapheur]\n' + f.read()
config_fp = io.BytesIO(config_string)
config_f = ConfigParser.RawConfigParser()
config_f.readfp(config_fp)

try:
    current_host = config_f.get("Parapheur", "parapheur.hostname")
except ConfigParser.NoOptionError:
    pprint.error("Impossible de récupérer la propriété ", False, '')
    pprint.error("parapheur.hostname", True)

    pprint.info("Vérifier la présence de cette propriété dans le fichier alfresco-global.properties")
    pprint.info("Si celle-ci n'est pas présente, la renseigner avec l'URL ", False, '')
    pprint.warning("ACTUELLE ", True, '')
    pprint.info("(et non la nouvelle)", False)
    exit(1)

new_host = config.get("Parapheur", "new_url")


def replace_hostname_in_file(file_to_handle):
    # Read in the file
    with open(file_to_handle, 'r') as f_read:
        filedata = f_read.read()

    # Replace the target string
    filedata = filedata.replace(current_host, new_host)

    # Write the file out again
    with open(file_to_handle, 'w') as f_write:
        f_write.write(filedata)


# NginX
replace_hostname_in_file("/etc/nginx/conf.d/parapheur.conf")
replace_hostname_in_file("/etc/nginx/conf.d/parapheur_ssl.conf")
# Tomcat
replace_hostname_in_file("/opt/iParapheur/tomcat/scripts/ctl.sh")
replace_hostname_in_file("/opt/iParapheur/tomcat/conf/Catalina/localhost/alfresco.xml")
replace_hostname_in_file("/opt/iParapheur/tomcat/shared/classes/alfresco-global.properties")
replace_hostname_in_file("/opt/iParapheur/tomcat/shared/classes/iparapheur-global.properties")
# WSDL
replace_hostname_in_file("/var/www/parapheur/alfresco/iparapheur.wsdl")

pprint.warning("ATTENTION ! Le certificat serveur configuré dans le fichier /etc/nginx/conf.d/parapheur_ssl.conf "
               "ne correspond potentiellement plus avec le nouveau nom du parapheur.\nIl convient de remplacer "
               "ce certificat (localisé dans le dossier /etc/nginx/ssl/) pour que le parapheur soit totalement "
               "fonctionnel.\n", True)

pprint.success("Propriétés à modifier dans le fichier de configuration /etc/nginx/conf.d/parapheur_ssl.conf :")
pprint.header("- ssl_certficiate /etc/nginx/ssl/certificat.pem;     # Partie publique")
pprint.header("- ssl_certficiate_key /etc/nginx/ssl/certificat.key; # Partie privée\n")

pprint.info("Une fois les modifications de certificat effectuées, relancer le service NginX :")
pprint.header("service nginx restart\n\n")

pprint.success("Modification de nom terminée.")



import os
import sys
from subprocess import check_call, CalledProcessError


def checkinstallation():
    import checkInstallationIP


def recuparchives():
    import recupArchives


def import_data():
    import import_data


def export_data():
    import export_data


def rename():
    import change_name


def remove_ldap():
    import remove_ldap


def pushdoc():
    import pushdoc


def properties_merger():
    args = sys.argv[1:]
    args.insert(0, os.path.dirname(os.path.realpath(__file__)) + "/shell/properties-merger/properties-merger.sh")
    try:
        check_call(args)
    except CalledProcessError:
        pass



# coding=utf-8

import parapheur
from progress.bar import IncrementalBar

client = parapheur.getrestclient()

if client.islogged:
    utilisateurs = client.doget("/parapheur/utilisateurs")
    bar = IncrementalBar('Analyse des utilisateurs', max=len(utilisateurs), suffix='%(index)d/%(max)d - %(eta)ds')

    to_delete = []

    for user in utilisateurs:
        if user["isFromLdap"]:
            bureaux = client.doget("/parapheur/utilisateurs/%s/bureaux" % user["id"], {"administres": "false"})
            if len(bureaux) == 0:
                to_delete.append(user)
                client.dodelete("/parapheur/utilisateurs/%s" % user["id"])

        bar.next()

    bar.finish()

    bar = IncrementalBar('Suppression des utilisateurs LDAP inutiles', max=len(to_delete), suffix='%(index)d/%(max)d - %(eta)ds')

    for user in to_delete:
        client.dodelete("/parapheur/utilisateurs/%s" % user["id"])
        bar.next()

    bar.finish()


#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

import parapheur

setup(
    name='iparapheur-utils.branches',
    version=parapheur.__version__,
    packages=find_packages(),
    author="Lukas Hameury",
    author_email="lukas.hameury@libriciel.fr",
    description="Client python pour i-Parapheur",
    long_description=open('README.md').read(),
    install_requires=[
        'suds',
        'requests>=2.16.0',
        'importlib',
        'PyMySql',
        'progress'
    ],
    include_package_data=True,
    url='https://gitlab.libriciel.fr/i-parapheur/client-python',
    entry_points={
        'console_scripts': [
            'ph-echo = parapheur.core:echo',
            'ph-check = parapheur.core:check',
            'ph-init = parapheur.core:init',
            'ph-recupArchives = parapheur.core:recuparchives',
            'ph-properties-merger = parapheur.core:properties_merger',
            'ph-export = parapheur.core:export_data',
            'ph-import = parapheur.core:import_data',
            'ph-rename = parapheur.core:rename',
            'ph-removeldap = parapheur.core:remove_ldap',
            'ph-pushdoc = parapheur.core:pushdoc'
        ],
    },
    license="CeCILL v2",
)
